{-------------------------------------------------------------------}
{ DSABSCTR.INC                                                      }
{ Copyright (c) 1993, 1994  Julian M. Bucknall. All Rights Reserved }
{-------------------------------------------------------------------}

{=TAbstractContainer==================================================
An abstract container

19Sep93 JMB
======================================================================}
constructor TAbstractContainer.Init(aNodeSize : word; DataOwner : boolean);
  begin
    if not inherited Init then Fail;
    InDone := false;
    Count := 0;
    NodeSize := aNodeSize;
    if (aNodeSize <> 0) then
      begin
        NS := GetNodeStore(aNodeSize);
        if not Assigned(NS) then
          begin
            ConstructorError := edsOutOfMemory;
            Fail;
          end;
      end;
    OwnsData := DataOwner;
    Compare := EZAbstractCompare;
    DisposeData := EZAbstractDisposeData;
    DupData := EZAbstractDupData;
  end;
{--------}
destructor TAbstractContainer.Done;
  begin
    InDone := true;
    Empty;
   {inherited Done;  <== no need for TObject ancestor}
    FreeNodeStore(NS);
  end;
{--------}
function  TAbstractContainer.Clone(DataOwner : boolean; NewCompare : TCompareFunc) : PAbstractContainer;
  begin
    Abstract;
  end;
{--------}
procedure TAbstractContainer.Empty;
  begin
    Abstract;
  end;
{--------}
procedure TAbstractContainer.Error(Code : word);
  var
    CodeStr : string[5];
  begin
    {$IFDEF DEBUG}
    case Code of
      edsOutOfMemory :
         HaltOnError('Memory allocation failed');
      edsTooManyItems :
         HaltOnError('Container has >64K items - MaxIs64KItems is on');
      edsInsInvalidHere :
         HaltOnError('Cannot insert at this node');
      edsDelInvalidHere :
         HaltOnError('Cannot delete this node');
      edsInsertDup :
         HaltOnError('Attempt to insert duplicate data');
      edsTreeStackError :
         HaltOnError('Stack problem with pre/in/postorder traversal - out of memory');
      edsTreeQueueError :
         HaltOnError('Queue problem with level order traversal - out of memory');
      edsCannotMoveHere :
         HaltOnError('Moving outside the container''s boundary');
      edsCloneError :
         HaltOnError('Error whilst cloning container: couldn''t dup data or out of memory');
      edsIncompatible :
         HaltOnError('Attempt to join two containers with different sized nodes');
    else
      begin
        Str(Code, CodeStr);
        HaltOnError('Unknown Error: ' + CodeStr);
      end;
    end;{case}
    {$ENDIF}
  end;
{--------}
function  TAbstractContainer.IsEmpty : boolean;
  begin
    IsEmpty := (Count = 0);
  end;
{--------}
function  TAbstractContainer.NewContainer(DataOwner : boolean) : PAbstractContainer;
  begin
    Abstract;
  end;
{--------}
procedure TAbstractContainer.SetCompare(NewFunc : TCompareFunc);
  begin
    if IsEmpty then
      if Assigned(NewFunc) then
           Compare := NewFunc
      else Compare := EZAbstractCompare;
  end;
{--------}
procedure TAbstractContainer.SetDisposeData(NewProc : TDisposeDataProc);
  begin
    if IsEmpty then
      if Assigned(NewProc) then
           DisposeData := NewProc
      else DisposeData := EZAbstractDisposeData;
  end;
{--------}
procedure TAbstractContainer.SetDupData(NewFunc : TDupDataFunc);
  begin
    if IsEmpty then
      if Assigned(NewFunc) then
           DupData := NewFunc
      else DupData := EZAbstractDupData;
  end;
{--------}
procedure TAbstractContainer.acDisposeNode(aNode : PNode);
  begin
    {$IFDEF DEBUG}
    Assert(Assigned(aNode), 'TAbstractContainer.acDisposeNode');
    Assert((NodeSize <> 0), 'TAbstractContainer.acDisposeNode 2');
    {$ENDIF}
    NS^.Dealloc(aNode);
    dec(Count);
  end;
{--------}
function  TAbstractContainer.acNewNode(aData : pointer) : PNode;
  var
    Node : PNode;
  begin
    {$IFDEF DEBUG}
    Assert((NodeSize <> 0), 'TAbstractContainer.acNewNode');
    {$ENDIF}
    Node := NS^.Alloc;
    if not Assigned(Node) then
      Error(edsOutOfMemory)
    else
      begin
        FillChar(Node^, NodeSize, 0);
        inc(Count);
        {$IFDEF MaxIs64KItems}
        if (Count = 0) then {overflowed word}
          begin
            acDisposeNode(Node);
            Error(edsTooManyItems);
            acNewNode := nil;
            Exit;
          end;
        {$ENDIF}
        Node^.Data := aData;
      end;
    acNewNode := Node;
  end;
{---------------------------------------------------------------------}
