{-------------------------------------------------------------------}
{ DSQUEUE.INC                                                       }
{ Copyright (c) 1993, 1994  Julian M. Bucknall. All Rights Reserved }
{-------------------------------------------------------------------}

{=TQueue==============================================================
A standard queue object.

19Sep93 JMB
======================================================================}
constructor TQueue.Init(DataOwner : boolean);
  begin
    if not inherited Init(8, DataOwner) then Fail;
    {This allocation always succeeds}
    Head := acNewNode(nil);
    Head^.Link := Head;
    Tail := Head;
    Count := 0;
  end;
{--------}
function TQueue.Append(aData : pointer) : boolean;
  var
    Node : PNode;
  begin
    Append := false;
    Node := acNewNode(aData);
    if Assigned(Node) then
      begin
        with Tail^ do
          begin
            Node^.Link := Link;
            Link := Node;
          end;
        Tail := Node;
        Append := true;
      end;
  end;
{--------}
function  TQueue.Clone(DataOwner : boolean;
                       NewCompare : TCompareFunc) : PAbstractContainer;
  var
    JoinNode,
    NewNode,
    Node     : PNode;
    NewQueue : PQueue;
    NewData  : pointer;
    DataError: boolean;
  begin
    NewQueue := PQueue(NewContainer(DataOwner));
    if not Assigned(NewQueue) then
      Error(ConstructorError)
    else
      begin
        NewQueue^.SetCompare(NewCompare);
        DataError := false;
        Node := Head^.Link;
        while (Node <> Head) and (not DataError) do
          begin
            if DataOwner then
              NewData := Node^.Data
            else
              DataError := not DupData(Node^.Data, NewData);
            if not DataError then
              begin
                if not NewQueue^.Append(acNewNode(NewData)) then
                  begin
                    if DataOwner then
                      DisposeData(NewData);
                    Dispose(NewQueue, Done);
                    Clone := nil;
                    Exit;
                  end;
                Node := Node^.Link;
              end;
          end;
        if DataError then
          begin
            Dispose(NewQueue, Done);
            NewQueue := nil;
            Error(edsCloneError);
          end;
      end;
    Clone := NewQueue;
  end;
{--------}
procedure TQueue.Empty;
  begin
    if OwnsData then
      while not IsEmpty do DisposeData(Pop)
    else
      while not IsEmpty do Pop;
    if InDone then
      acDisposeNode(Head);
  end;
{--------}
function  TQueue.Examine : pointer;
  begin
    {$IFDEF DEBUG}
    Assert(not IsEmpty, 'TQueue.Examine');
    {$ENDIF}
    Examine := Head^.Link^.Data;
  end;
{--------}
function  TQueue.NewContainer(DataOwner : boolean) : PAbstractContainer;
  var
    NC : PAbstractContainer;
  begin
    NC := New(PQueue, Init(DataOwner));
    if Assigned(NC) then
      begin
        NC^.SetCompare(Compare);
        NC^.SetDisposeData(DisposeData);
        NC^.SetDupData(DupData);
      end;
    NewContainer := NC;
  end;
{--------}
function  TQueue.Pop : pointer;
  var
    Node : PNode;
  begin
    {$IFDEF DEBUG}
    Assert(not IsEmpty, 'TQueue.Pop');
    {$ENDIF}
    Node := Head^.Link;
    Head^.Link := Node^.Link;
    Pop := Node^.Data;
    acDisposeNode(Node);
    if IsEmpty then Tail := Head;
  end;
{---------------------------------------------------------------------}


{=TDeque==============================================================
A output restricted deque object.

This type of deque allows queue jumpers, ie data objects can also be
pushed into the front of the queue, giving it stack-like behaviour. It
is descended from the basic queue and inherits Pop and Append.

Methods
  Push       pushes the data object aData to the front of the deque

19Sep93 JMB
======================================================================}
function  TDeque.NewContainer(DataOwner : boolean) : PAbstractContainer;
  var
    NC : PAbstractContainer;
  begin
    NC := New(PDeque, Init(DataOwner));
    if Assigned(NC) then
      begin
        NC^.SetCompare(Compare);
        NC^.SetDisposeData(DisposeData);
        NC^.SetDupData(DupData);
      end;
    NewContainer := NC;
  end;
{--------}
function  TDeque.Push(aData : pointer) : boolean;
  var
    Node : PNode;
  begin
    Push := false;
    Node := acNewNode(aData);
    if Assigned(Node) then
      begin
        with Head^ do
          begin
            Node^.Link := Link;
            Link := Node;
          end;
        if (Tail = Head) then
          Tail := Node;
        Push := true;
      end;
  end;
{---------------------------------------------------------------------}
